<?php

namespace Vialibri\HarvestPlugin\CustomPostTypes;

class SettingsTab {

  public static function init() {
    register_setting(
      'vharvest_cpt_settings',
      'vharvest_cpt_settings'
    );
  }

  public static function enqueueScript($hook) {
    global $vharvest_cpt_plugin_version;

    if($hook === 'settings_page_vialibri-harvest-settings'
        && \Vialibri\HarvestPlugin\SettingsPage::currentTabSlug() === 'custom-post-types') {

      wp_enqueue_script(
        'vialibri-harvest-cpt-settings-script',
        plugins_url('/../assets/js/admin.js', __FILE__),
        array('jquery'),
        $vharvest_cpt_plugin_version
      );
    }
  }

  public static function addTab(array $tabs) {
    return array_merge( array( 'custom-post-types' => 'Custom Post Types'), $tabs);
  }

  public static function renderTab() {
    add_settings_section(
      'vharvest_cpt',
      'Custom Post Types',
      '__return_empty_string',
      'vialibri-harvest-settings'
    );
    add_settings_field(
      'post_type',
      'Post type',
      array('Vialibri\HarvestPlugin\CustomPostTypes\SettingsTab', 'renderPostTypeField'),
      'vialibri-harvest-settings',
      'custom-post-types'
    );

    foreach (self::getFields() as $key => $details) {
      add_settings_field(
        'mapping_'.$key,
        'Field for: '. $details['label'] . ($details['required'] ? ' (required)' : ''),
        array('Vialibri\HarvestPlugin\CustomPostTypes\SettingsTab', 'renderMappingField'),
        'vialibri-harvest-settings',
        'custom-post-types',
        array(
          'key' => 'mapping_'.$key,
          'allow_multiple' => $details['allow_multiple'] ?? true,
          'required' => $details['required'],
        )
      );
    }
    ?>

    <style type="text/css">

      select.vialibri-harvest-cpt-subfield {
        vertical-align: baseline;
      }

      .vialibri-harvest-cpt-delete {
        color: #a00;
        cursor: pointer;
        margin-left: 10px;
        font-size: 80%;
      }

      .vialibri-harvest-cpt-delete:hover {
        color: #dc3232;
      }
    </style>

    <form method="POST" action="options.php">
      <?php settings_fields('vharvest_cpt_settings'); ?>
      <table class="form-table">
        <?php do_settings_fields('vialibri-harvest-settings', 'custom-post-types'); ?>
      </table>
      <input name="Submit" type="submit" class="button-primary" value="Save Changes" />
    </form>

    <?php
  }

  public static function renderPostTypeField(array $args) {
    $key = 'post_type';
    $options = get_option('vharvest_cpt_settings');
    $value = isset($options[$key]) ? $options[$key] : '';

    $options = array();
    foreach (get_post_types(array( 'public' => true, '_builtin' => false ), 'objects') as $type) {
      $options[$type->name] = $type->labels->name;
    }

    echo self::formatSelect($key, $value, $options);
    echo '<div id="post-type-spinner" class="spinner is-active" style="float: none; margin-top: 0px;"></div>';
  }

  public static function renderMappingField(array $args) {
    $key = $args['key'];
    $options = get_option('vharvest_cpt_settings');
    $value = isset($options[$key]) ? $options[$key] : '';

    $extras = array(
      'disabled' => 'disabled',
      'data-current-value' => $value,
      'data-allow-multiple' => $args['allow_multiple'] ? 'true' : 'false'
    );

    if($args['required']) {
      $extras['required'] = 'required';
    }

    echo self::formatSelect($key, $value, array(), $extras);

    // And now the div and input for use when we're dealing with multiple fields.
    $fieldsKey = $key . '_subfields';
    $fieldsValue = isset($options[$fieldsKey]) ? $options[$fieldsKey] : '[]';
    echo '<div class="vialibri-harvest-cpt-subfields" data-field="'.$key.'" data-subfields="'.$fieldsKey.'" style="display: none;"><ol></ol><input type="button" value="Add field" class="button"></div>';
    echo '<input id="'.$fieldsKey.'" name="vharvest_cpt_settings['.$fieldsKey.']" value="'.esc_attr($fieldsValue).'" type="hidden" />';
  }

  public static function ajaxMetaKeysForPostType() {
    // Taken from https://wordpress.stackexchange.com/a/58840/38105.
    global $wpdb;
    $postType = $_GET['post_type'];
    $query = "
      SELECT DISTINCT($wpdb->postmeta.meta_key)
      FROM $wpdb->posts
      INNER JOIN $wpdb->postmeta
        ON $wpdb->posts.ID = $wpdb->postmeta.post_id
      WHERE $wpdb->posts.post_type = '%s'
        AND $wpdb->postmeta.meta_key != ''
        AND $wpdb->postmeta.meta_key NOT RegExp '(^[0-9]+$)'
        AND $wpdb->postmeta.meta_key NOT IN ('_edit_last', '_edit_lock')
      ORDER BY $wpdb->postmeta.meta_key
    ";
    $metaKeys = $wpdb->get_col($wpdb->prepare($query, $postType));

    $acfFields = [];
    if (class_exists('ACF')) {
      // ACF fields show up twice in postmeta table. Once with an underscore in front and once without.
      // The underscored version contains the field ID.
      $query = "
        SELECT pm.meta_key `name`,
          pm_u.meta_value `key`
        FROM $wpdb->posts p
        INNER JOIN $wpdb->postmeta pm
          ON p.ID = pm.post_id
        INNER JOIN $wpdb->postmeta pm_u
          ON p.ID = pm_u.post_id
          AND pm_u.meta_key = CONCAT('_', pm.meta_key)
        WHERE p.post_type = '%s'
        AND pm.meta_key != ''
        AND pm_u.meta_value LIKE 'field_%'
        GROUP BY pm.meta_key,
          pm.meta_value
        ORDER BY pm.meta_key
      ";
      $acfDbFields = $wpdb->get_results($wpdb->prepare($query, $postType), ARRAY_A);

      // Fetch the "user friendly" name of the field.
      $acfLabels = [];
      foreach ($acfDbFields as $acfDbField) {
        $fieldData = get_field_object($acfDbField['key']);
        $acfLabels[$acfDbField['name']] = $fieldData['label'];
      }

      foreach ($acfDbFields as $acfDbField) {
        // Look for fields with duplicate labels and include the field name in their label.
        $label = $acfLabels[$acfDbField['name']];
        $labelCount = count(array_filter($acfLabels, function ($l) use ($label) { return $l === $label; }));

        $acfFields[$acfDbField['name']] = $label . ($labelCount > 1 ? ' (' . $acfDbField['name'] . ')' : '');

        // The ACF fields also show up in the meta keys. We want to stop that.
        // They also show up with an underscore in front.
        $metaKeys = array_values(array_diff($metaKeys, [
          $acfDbField['name'],
          '_' . $acfDbField['name']
        ]));
      }

      asort($acfFields);
    }

    echo json_encode([
      'metaKeys' => $metaKeys,
      'acfFields' => $acfFields
    ]);
    exit;
  }

  public static function getFields() {
    return array(
      'author' => array(
        'label' => 'Author',
        'required' => false,
        'type' => 'text'
      ),
      'title' => array(
        'label' => 'Title',
        'required' => true,
        'type' => 'text'
      ),
      'description' => array(
        'label' => 'Description',
        'required' => false,
        'type' => 'text'
      ),
      'year' => array(
        'label' => 'Year of publication',
        'required' => false,
        'type' => 'text',
        'allow_multiple' => false
      ),
      'publisher' => array(
        'label' => 'Publisher',
        'required' => false,
        'type' => 'text',
        'allow_multiple' => false
      ),
      'price' => array(
        'label' => 'Price',
        'required' => true,
        'type' => 'text',
        'allow_multiple' => false
      ),
      'image_url' => array(
        'label' => 'Image',
        'required' => false,
        'type' => 'media',
        'allow_multiple' => false
      ),
      'edition' => array(
        'label' => 'Edition',
        'required' => false,
        'type' => 'text'
      ),
      'sku_dealer_item_id' => array(
        'label' => 'Dealer\'s ID',
        'required' => false,
        'type' => 'text',
        'allow_multiple' => false
      ),
      'isbn' => array(
        'label' => 'ISBN',
        'required' => false,
        'type' => 'text',
        'allow_multiple' => false
      )
    );
  }

  private static function formatOption($value, $text, $selected = false) {
    $option = '<option value="'.esc_attr($value).'"';
    if($selected) {
      $option .= ' selected="selected"';
    }
    return $option.'>'.esc_html($text).'</option>';
  }

  private static function formatSelect($key, $currentValue, array $options, $extraValues = []) {
    $extras = '';
    foreach ($extraValues as $extraKey => $extraValue) {
      $extras .= ' '.$extraKey.'="'.esc_attr($extraValue).'"';
    }
    $inputHtml = '<select id="'.$key.'" name="vharvest_cpt_settings['.$key.']" class="regular-text" '.$extras.'>';
    foreach ($options as $value => $text) {
      $inputHtml .= self::formatOption($value, $text, $value == $currentValue );
    }
    $inputHtml .= '</select>';

    return $inputHtml;
  }

}
