jQuery(function($) {
  var optionsByPostType = {};

  function init() {
    var mappingSelects = $('select[name^="vharvest_cpt_settings[mapping_"]');
    mappingSelects.change(function() {
      updateSubfieldDisplay(this.id, $(this).val() === 'multiple');
    });

    updateFieldSelectors(function() {
      mappingSelects.each(function() {
        updateSubfieldDisplay(this.id, $(this).val() === 'multiple');
      });
    });
    $('select[name="vharvest_cpt_settings[post_type]"]').change(updateFieldSelectors);

    $('.vialibri-harvest-cpt-subfields').on('change', '.vialibri-harvest-cpt-subfield', function() {
      updateHiddenInputFromSubfields($(this).closest('.vialibri-harvest-cpt-subfields'));
    });

    $('.vialibri-harvest-cpt-subfields input[type="button"]').click(function() {
      addSubfield('', $(this).closest('.vialibri-harvest-cpt-subfields'), true);
      updateFieldSelectors();
    });

    $('.vialibri-harvest-cpt-subfields').on('click', '.vialibri-harvest-cpt-delete', function() {
      removeSubfield($(this).closest('li'));
    });
  }

  function ensureOptions(callback) {
    var postType = currentPostType();

    if (optionsByPostType[postType]) {
      callback(optionsByPostType[postType]);
    } else {
      var requestData = {
        action: 'vialibri_harvest_cpt_post_types',
        post_type: postType
      };
      var spinner = $('#post-type-spinner').show();

      jQuery.getJSON(ajaxurl, requestData, function(data) {
        var options = [
          { key: '', name: '(No matching field)' },
          { key: 'multiple', name: 'Multiple fields' },
          { key: 'post:title', name: 'Post field: Title' },
          { key: 'post:content', name: 'Post field: Content' },
          { key: 'post:excerpt', name: 'Post field: Excerpt' },
          { key: 'post:featured_image', name: 'Post field: Featured Image' }
        ];
        options = options.concat(
          $.map(data.acfFields, function(fieldLabel, fieldName) {
            return { key: 'acf:' + fieldName, name: 'ACF field: ' + fieldLabel };
          })
        );
        options = options.concat(
          $.map(data.metaKeys, function(field) {
            return { key: 'meta:' + field, name: 'Meta field: ' + field };
          })
        );

        optionsByPostType[postType] = options;
        spinner.hide();

        callback(options);
      });
    }
  }

  function currentPostType() {
    return $('select[name="vharvest_cpt_settings[post_type]"]').val();
  }

  function updateFieldSelectors(callback) {
    ensureOptions(function(options) {
      var optionsWithoutMultiple = $.grep(options, function(o) {
        return o.key !== 'multiple';
      });

      $('select[name^="vharvest_cpt_settings[mapping_"], select.vialibri-harvest-cpt-subfield')
        .attr('disabled', true)
        .each(function() {
          populateList(
            $(this),
            !$(this).data('allowMultiple') || $(this).hasClass('vialibri-harvest-cpt-subfield')
              ? optionsWithoutMultiple
              : options
          );
        });

      if (callback && callback instanceof Function) {
        callback();
      }
    });
  }

  function populateList(el, options) {
    var currentValue = el.val();
    if (currentValue === null) {
      // First load. Take it from the data value instead.
      currentValue = el.data('currentValue');

      // In 0.4.0 we started loading ACF fields specifically where we had meta fields before. Try to convert these.
      if (currentValue && currentValue.indexOf('meta:') === 0) {
        var metaCount = $.grep(options, function(opt) {
          return opt.key === currentValue;
        }).length;
        var acfEquivalent = currentValue.replace('meta:', 'acf:');
        var acfCount = $.grep(options, function(opt) {
          return opt.key === acfEquivalent;
        }).length;

        if (metaCount === 0 && acfCount === 1) {
          currentValue = acfEquivalent;
        }
      }
    }
    el.empty();

    el.append(
      $.map(options, function(opt) {
        return $('<option></option>')
          .attr('value', opt.key)
          .text(opt.name)
          .prop('selected', opt.key === currentValue);
      })
    );
    el.attr('disabled', false);
  }

  function updateSubfieldDisplay(field, display) {
    var container = $('.vialibri-harvest-cpt-subfields[data-field="' + field + '"]');

    if (display) {
      // Check if it's already been initialised.
      if (container.children('ol').children().length === 0) {
        var existingValues = JSON.parse($('#' + container.data('subfields')).val());
        addSubfield(existingValues[0] || '', container, false);
        addSubfield(existingValues[1] || '', container, false);

        var remaining = existingValues.slice(2);
        for (var i = 0; i < remaining.length; i++) {
          addSubfield(remaining[i], container, true);
        }

        updateFieldSelectors();
      }

      container.show();
    } else {
      container
        .children('ol')
        .children()
        .remove();

      container.hide();
    }
  }

  function addSubfield(currentValue, container, canBeRemoved) {
    var li = $('<li></li>');

    li.append(
      $('<select></select>')
        .addClass('vialibri-harvest-cpt-subfield')
        .prop('required', !canBeRemoved)
        .attr('data-current-value', currentValue)
    );

    if (canBeRemoved) {
      li.append(
        $('<a></a>')
          .addClass('vialibri-harvest-cpt-delete')
          .text('Remove')
      );
    }

    container.children('ol').append(li);
  }

  function removeSubfield(li) {
    var container = li.closest('.vialibri-harvest-cpt-subfields');

    li.remove();

    updateHiddenInputFromSubfields(container);
  }

  function updateHiddenInputFromSubfields(container) {
    var fields = container
      .find('.vialibri-harvest-cpt-subfield')
      .map(function(i, el) {
        return $(el).val();
      })
      .filter(function(i, val) {
        return val !== '';
      })
      .toArray();

    $('#' + container.data('subfields')).val(JSON.stringify(fields));
  }

  init();
});
